[Python-checkins] bpo-36419: IDLE - Refactor autocompete and improve testing. (#15121)

Terry Jan Reedy webhook-mailer at python.org
Sun Aug 4 19:49:02 EDT 2019


https://github.com/python/cpython/commit/1213123005d9f94bb5027c0a5256ea4d3e97b61d
commit: 1213123005d9f94bb5027c0a5256ea4d3e97b61d
branch: master
author: Terry Jan Reedy <tjreedy at udel.edu>
committer: GitHub <noreply at github.com>
date: 2019-08-04T19:48:52-04:00
summary:

bpo-36419: IDLE - Refactor autocompete and improve testing. (#15121)

files:
A Misc/NEWS.d/next/IDLE/2019-08-04-17-10-01.bpo-36419.TJZqOc.rst
M Lib/idlelib/autocomplete.py
M Lib/idlelib/autocomplete_w.py
M Lib/idlelib/idle_test/test_autocomplete.py

diff --git a/Lib/idlelib/autocomplete.py b/Lib/idlelib/autocomplete.py
index e20b757d8710..c623d45a1534 100644
--- a/Lib/idlelib/autocomplete.py
+++ b/Lib/idlelib/autocomplete.py
@@ -8,42 +8,45 @@
 import string
 import sys
 
-# These constants represent the two different types of completions.
-# They must be defined here so autocomple_w can import them.
-COMPLETE_ATTRIBUTES, COMPLETE_FILES = range(1, 2+1)
-
+# Two types of completions; defined here for autocomplete_w import below.
+ATTRS, FILES = 0, 1
 from idlelib import autocomplete_w
 from idlelib.config import idleConf
 from idlelib.hyperparser import HyperParser
 
+# Tuples passed to open_completions.
+#       EvalFunc, Complete, WantWin, Mode
+FORCE = True,     False,    True,    None   # Control-Space.
+TAB   = False,    True,     True,    None   # Tab.
+TRY_A = False,    False,    False,   ATTRS  # '.' for attributes.
+TRY_F = False,    False,    False,   FILES  # '/' in quotes for file name.
+
 # This string includes all chars that may be in an identifier.
 # TODO Update this here and elsewhere.
 ID_CHARS = string.ascii_letters + string.digits + "_"
 
-SEPS = os.sep
-if os.altsep:  # e.g. '/' on Windows...
-    SEPS += os.altsep
-
+SEPS = f"{os.sep}{os.altsep if os.altsep else ''}"
+TRIGGERS = f".{SEPS}"
 
 class AutoComplete:
 
     def __init__(self, editwin=None):
         self.editwin = editwin
-        if editwin is not None:   # not in subprocess or test
+        if editwin is not None:   # not in subprocess or no-gui test
             self.text = editwin.text
-            self.autocompletewindow = None
-            # id of delayed call, and the index of the text insert when
-            # the delayed call was issued. If _delayed_completion_id is
-            # None, there is no delayed call.
-            self._delayed_completion_id = None
-            self._delayed_completion_index = None
+        self.autocompletewindow = None
+        # id of delayed call, and the index of the text insert when
+        # the delayed call was issued. If _delayed_completion_id is
+        # None, there is no delayed call.
+        self._delayed_completion_id = None
+        self._delayed_completion_index = None
 
     @classmethod
     def reload(cls):
         cls.popupwait = idleConf.GetOption(
             "extensions", "AutoComplete", "popupwait", type="int", default=0)
 
-    def _make_autocomplete_window(self):
+    def _make_autocomplete_window(self):  # Makes mocking easier.
         return autocomplete_w.AutoCompleteWindow(self.text)
 
     def _remove_autocomplete_window(self, event=None):
@@ -52,30 +55,12 @@ def _remove_autocomplete_window(self, event=None):
             self.autocompletewindow = None
 
     def force_open_completions_event(self, event):
-        """Happens when the user really wants to open a completion list, even
-        if a function call is needed.
-        """
-        self.open_completions(True, False, True)
+        "(^space) Open completion list, even if a function call is needed."
+        self.open_completions(FORCE)
         return "break"
 
-    def try_open_completions_event(self, event):
-        """Happens when it would be nice to open a completion list, but not
-        really necessary, for example after a dot, so function
-        calls won't be made.
-        """
-        lastchar = self.text.get("insert-1c")
-        if lastchar == ".":
-            self._open_completions_later(False, False, False,
-                                         COMPLETE_ATTRIBUTES)
-        elif lastchar in SEPS:
-            self._open_completions_later(False, False, False,
-                                         COMPLETE_FILES)
-
     def autocomplete_event(self, event):
-        """Happens when the user wants to complete his word, and if necessary,
-        open a completion list after that (if there is more than one
-        completion)
-        """
+        "(tab) Complete word or open list if multiple options."
         if hasattr(event, "mc_state") and event.mc_state or\
                 not self.text.get("insert linestart", "insert").strip():
             # A modifier was pressed along with the tab or
@@ -85,34 +70,34 @@ def autocomplete_event(self, event):
             self.autocompletewindow.complete()
             return "break"
         else:
-            opened = self.open_completions(False, True, True)
+            opened = self.open_completions(TAB)
             return "break" if opened else None
 
-    def _open_completions_later(self, *args):
-        self._delayed_completion_index = self.text.index("insert")
-        if self._delayed_completion_id is not None:
-            self.text.after_cancel(self._delayed_completion_id)
-        self._delayed_completion_id = \
-            self.text.after(self.popupwait, self._delayed_open_completions,
-                            *args)
-
-    def _delayed_open_completions(self, *args):
+    def try_open_completions_event(self, event=None):
+        "(./) Open completion list after pause with no movement."
+        lastchar = self.text.get("insert-1c")
+        if lastchar in TRIGGERS:
+            args = TRY_A if lastchar == "." else TRY_F
+            self._delayed_completion_index = self.text.index("insert")
+            if self._delayed_completion_id is not None:
+                self.text.after_cancel(self._delayed_completion_id)
+            self._delayed_completion_id = self.text.after(
+                self.popupwait, self._delayed_open_completions, args)
+
+    def _delayed_open_completions(self, args):
+        "Call open_completions if index unchanged."
         self._delayed_completion_id = None
         if self.text.index("insert") == self._delayed_completion_index:
-            self.open_completions(*args)
+            self.open_completions(args)
 
-    def open_completions(self, evalfuncs, complete, userWantsWin, mode=None):
+    def open_completions(self, args):
         """Find the completions and create the AutoCompleteWindow.
         Return True if successful (no syntax error or so found).
         If complete is True, then if there's nothing to complete and no
         start of completion, won't open completions and return False.
         If mode is given, will open a completion list only in this mode.
-
-        Action  Function                Eval   Complete WantWin Mode
-        ^space  force_open_completions  True,  False,   True    no
-        . or /  try_open_completions    False, False,   False   yes
-        tab     autocomplete            False, True,    True    no
         """
+        evalfuncs, complete, wantwin, mode = args
         # Cancel another delayed call, if it exists.
         if self._delayed_completion_id is not None:
             self.text.after_cancel(self._delayed_completion_id)
@@ -121,14 +106,14 @@ def open_completions(self, evalfuncs, complete, userWantsWin, mode=None):
         hp = HyperParser(self.editwin, "insert")
         curline = self.text.get("insert linestart", "insert")
         i = j = len(curline)
-        if hp.is_in_string() and (not mode or mode==COMPLETE_FILES):
+        if hp.is_in_string() and (not mode or mode==FILES):
             # Find the beginning of the string.
             # fetch_completions will look at the file system to determine
             # whether the string value constitutes an actual file name
             # XXX could consider raw strings here and unescape the string
             # value if it's not raw.
             self._remove_autocomplete_window()
-            mode = COMPLETE_FILES
+            mode = FILES
             # Find last separator or string start
             while i and curline[i-1] not in "'\"" + SEPS:
                 i -= 1
@@ -138,17 +123,17 @@ def open_completions(self, evalfuncs, complete, userWantsWin, mode=None):
             while i and curline[i-1] not in "'\"":
                 i -= 1
             comp_what = curline[i:j]
-        elif hp.is_in_code() and (not mode or mode==COMPLETE_ATTRIBUTES):
+        elif hp.is_in_code() and (not mode or mode==ATTRS):
             self._remove_autocomplete_window()
-            mode = COMPLETE_ATTRIBUTES
+            mode = ATTRS
             while i and (curline[i-1] in ID_CHARS or ord(curline[i-1]) > 127):
                 i -= 1
             comp_start = curline[i:j]
-            if i and curline[i-1] == '.':
+            if i and curline[i-1] == '.':  # Need object with attributes.
                 hp.set_index("insert-%dc" % (len(curline)-(i-1)))
                 comp_what = hp.get_expression()
-                if not comp_what or \
-                   (not evalfuncs and comp_what.find('(') != -1):
+                if (not comp_what or
+                   (not evalfuncs and comp_what.find('(') != -1)):
                     return None
             else:
                 comp_what = ""
@@ -163,7 +148,7 @@ def open_completions(self, evalfuncs, complete, userWantsWin, mode=None):
         self.autocompletewindow = self._make_autocomplete_window()
         return not self.autocompletewindow.show_window(
                 comp_lists, "insert-%dc" % len(comp_start),
-                complete, mode, userWantsWin)
+                complete, mode, wantwin)
 
     def fetch_completions(self, what, mode):
         """Return a pair of lists of completions for something. The first list
@@ -185,7 +170,7 @@ def fetch_completions(self, what, mode):
             return rpcclt.remotecall("exec", "get_the_completion_list",
                                      (what, mode), {})
         else:
-            if mode == COMPLETE_ATTRIBUTES:
+            if mode == ATTRS:
                 if what == "":
                     namespace = {**__main__.__builtins__.__dict__,
                                  **__main__.__dict__}
@@ -207,7 +192,7 @@ def fetch_completions(self, what, mode):
                     except:
                         return [], []
 
-            elif mode == COMPLETE_FILES:
+            elif mode == FILES:
                 if what == "":
                     what = "."
                 try:
diff --git a/Lib/idlelib/idle_test/test_autocomplete.py b/Lib/idlelib/idle_test/test_autocomplete.py
index 6181b29ec250..2c478cd5c2a1 100644
--- a/Lib/idlelib/idle_test/test_autocomplete.py
+++ b/Lib/idlelib/idle_test/test_autocomplete.py
@@ -1,4 +1,4 @@
-"Test autocomplete, coverage 87%."
+"Test autocomplete, coverage 93%."
 
 import unittest
 from unittest.mock import Mock, patch
@@ -45,127 +45,177 @@ def setUp(self):
 
     def test_init(self):
         self.assertEqual(self.autocomplete.editwin, self.editor)
+        self.assertEqual(self.autocomplete.text, self.text)
 
     def test_make_autocomplete_window(self):
         testwin = self.autocomplete._make_autocomplete_window()
         self.assertIsInstance(testwin, acw.AutoCompleteWindow)
 
     def test_remove_autocomplete_window(self):
-        self.autocomplete.autocompletewindow = (
-            self.autocomplete._make_autocomplete_window())
-        self.autocomplete._remove_autocomplete_window()
-        self.assertIsNone(self.autocomplete.autocompletewindow)
+        acp = self.autocomplete
+        acp.autocompletewindow = m = Mock()
+        acp._remove_autocomplete_window()
+        m.hide_window.assert_called_once()
+        self.assertIsNone(acp.autocompletewindow)
 
     def test_force_open_completions_event(self):
-        # Test that force_open_completions_event calls _open_completions.
-        o_cs = Func()
-        self.autocomplete.open_completions = o_cs
-        self.autocomplete.force_open_completions_event('event')
-        self.assertEqual(o_cs.args, (True, False, True))
-
-    def test_try_open_completions_event(self):
-        Equal = self.assertEqual
-        autocomplete = self.autocomplete
-        trycompletions = self.autocomplete.try_open_completions_event
-        o_c_l = Func()
-        autocomplete._open_completions_later = o_c_l
-
-        # _open_completions_later should not be called with no text in editor.
-        trycompletions('event')
-        Equal(o_c_l.args, None)
-
-        # _open_completions_later should be called with COMPLETE_ATTRIBUTES (1).
-        self.text.insert('1.0', 're.')
-        trycompletions('event')
-        Equal(o_c_l.args, (False, False, False, 1))
-
-        # _open_completions_later should be called with COMPLETE_FILES (2).
-        self.text.delete('1.0', 'end')
-        self.text.insert('1.0', '"./Lib/')
-        trycompletions('event')
-        Equal(o_c_l.args, (False, False, False, 2))
+        # Call _open_completions and break.
+        acp = self.autocomplete
+        open_c = Func()
+        acp.open_completions = open_c
+        self.assertEqual(acp.force_open_completions_event('event'), 'break')
+        self.assertEqual(open_c.args[0], ac.FORCE)
 
     def test_autocomplete_event(self):
         Equal = self.assertEqual
-        autocomplete = self.autocomplete
+        acp = self.autocomplete
 
-        # Test that the autocomplete event is ignored if user is pressing a
-        # modifier key in addition to the tab key.
+        # Result of autocomplete event: If modified tab, None.
         ev = Event(mc_state=True)
-        self.assertIsNone(autocomplete.autocomplete_event(ev))
+        self.assertIsNone(acp.autocomplete_event(ev))
         del ev.mc_state
 
-        # Test that tab after whitespace is ignored.
+        # If tab after whitespace, None.
         self.text.insert('1.0', '        """Docstring.\n    ')
-        self.assertIsNone(autocomplete.autocomplete_event(ev))
+        self.assertIsNone(acp.autocomplete_event(ev))
         self.text.delete('1.0', 'end')
 
-        # If autocomplete window is open, complete() method is called.
+        # If active autocomplete window, complete() and 'break'.
         self.text.insert('1.0', 're.')
-        # This must call autocomplete._make_autocomplete_window().
-        Equal(self.autocomplete.autocomplete_event(ev), 'break')
-
-        # If autocomplete window is not active or does not exist,
-        # open_completions is called. Return depends on its return.
-        autocomplete._remove_autocomplete_window()
-        o_cs = Func()  # .result = None.
-        autocomplete.open_completions = o_cs
-        Equal(self.autocomplete.autocomplete_event(ev), None)
-        Equal(o_cs.args, (False, True, True))
-        o_cs.result = True
-        Equal(self.autocomplete.autocomplete_event(ev), 'break')
-        Equal(o_cs.args, (False, True, True))
-
-    def test_open_completions_later(self):
-        # Test that autocomplete._delayed_completion_id is set.
+        acp.autocompletewindow = mock = Mock()
+        mock.is_active = Mock(return_value=True)
+        Equal(acp.autocomplete_event(ev), 'break')
+        mock.complete.assert_called_once()
+        acp.autocompletewindow = None
+
+        # If no active autocomplete window, open_completions(), None/break.
+        open_c = Func(result=False)
+        acp.open_completions = open_c
+        Equal(acp.autocomplete_event(ev), None)
+        Equal(open_c.args[0], ac.TAB)
+        open_c.result = True
+        Equal(acp.autocomplete_event(ev), 'break')
+        Equal(open_c.args[0], ac.TAB)
+
+    def test_try_open_completions_event(self):
+        Equal = self.assertEqual
+        text = self.text
         acp = self.autocomplete
+        trycompletions = acp.try_open_completions_event
+        after = Func(result='after1')
+        acp.text.after = after
+
+        # If no text or trigger, after not called.
+        trycompletions()
+        Equal(after.called, 0)
+        text.insert('1.0', 're')
+        trycompletions()
+        Equal(after.called, 0)
+
+        # Attribute needed, no existing callback.
+        text.insert('insert', ' re.')
         acp._delayed_completion_id = None
-        acp._open_completions_later(False, False, False, ac.COMPLETE_ATTRIBUTES)
+        trycompletions()
+        Equal(acp._delayed_completion_index, text.index('insert'))
+        Equal(after.args,
+              (acp.popupwait, acp._delayed_open_completions, ac.TRY_A))
         cb1 = acp._delayed_completion_id
-        self.assertTrue(cb1.startswith('after'))
-
-        # Test that cb1 is cancelled and cb2 is new.
-        acp._open_completions_later(False, False, False, ac.COMPLETE_FILES)
-        self.assertNotIn(cb1, self.root.tk.call('after', 'info'))
-        cb2 = acp._delayed_completion_id
-        self.assertTrue(cb2.startswith('after') and cb2 != cb1)
-        self.text.after_cancel(cb2)
+        Equal(cb1, 'after1')
+
+        # File needed, existing callback cancelled.
+        text.insert('insert', ' "./Lib/')
+        after.result = 'after2'
+        cancel = Func()
+        acp.text.after_cancel = cancel
+        trycompletions()
+        Equal(acp._delayed_completion_index, text.index('insert'))
+        Equal(cancel.args, (cb1,))
+        Equal(after.args,
+              (acp.popupwait, acp._delayed_open_completions, ac.TRY_F))
+        Equal(acp._delayed_completion_id, 'after2')
 
     def test_delayed_open_completions(self):
-        # Test that autocomplete._delayed_completion_id set to None
-        # and that open_completions is not called if the index is not
-        # equal to _delayed_completion_index.
+        Equal = self.assertEqual
         acp = self.autocomplete
-        acp.open_completions = Func()
+        open_c = Func()
+        acp.open_completions = open_c
+        self.text.insert('1.0', '"dict.')
+
+        # Set autocomplete._delayed_completion_id to None.
+        # Text index changed, don't call open_completions.
         acp._delayed_completion_id = 'after'
         acp._delayed_completion_index = self.text.index('insert+1c')
-        acp._delayed_open_completions(1, 2, 3)
+        acp._delayed_open_completions('dummy')
         self.assertIsNone(acp._delayed_completion_id)
-        self.assertEqual(acp.open_completions.called, 0)
+        Equal(open_c.called, 0)
 
-        # Test that open_completions is called if indexes match.
+        # Text index unchanged, call open_completions.
         acp._delayed_completion_index = self.text.index('insert')
-        acp._delayed_open_completions(1, 2, 3, ac.COMPLETE_FILES)
-        self.assertEqual(acp.open_completions.args, (1, 2, 3, 2))
+        acp._delayed_open_completions((1, 2, 3, ac.FILES))
+        self.assertEqual(open_c.args[0], (1, 2, 3, ac.FILES))
+
+    def test_oc_cancel_comment(self):
+        none = self.assertIsNone
+        acp = self.autocomplete
+
+        # Comment is in neither code or string.
+        acp._delayed_completion_id = 'after'
+        after = Func(result='after')
+        acp.text.after_cancel = after
+        self.text.insert(1.0, '# comment')
+        none(acp.open_completions(ac.TAB))  # From 'else' after 'elif'.
+        none(acp._delayed_completion_id)
+
+    def test_oc_no_list(self):
+        acp = self.autocomplete
+        fetch = Func(result=([],[]))
+        acp.fetch_completions = fetch
+        self.text.insert('1.0', 'object')
+        self.assertIsNone(acp.open_completions(ac.TAB))
+        self.text.insert('insert', '.')
+        self.assertIsNone(acp.open_completions(ac.TAB))
+        self.assertEqual(fetch.called, 2)
+
+
+    def test_open_completions_none(self):
+        # Test other two None returns.
+        none = self.assertIsNone
+        acp = self.autocomplete
+
+        # No object for attributes or need call not allowed.
+        self.text.insert(1.0, '.')
+        none(acp.open_completions(ac.TAB))
+        self.text.insert('insert', ' int().')
+        none(acp.open_completions(ac.TAB))
+
+        # Blank or quote trigger 'if complete ...'.
+        self.text.delete(1.0, 'end')
+        self.assertFalse(acp.open_completions(ac.TAB))
+        self.text.insert('1.0', '"')
+        self.assertFalse(acp.open_completions(ac.TAB))
+        self.text.delete('1.0', 'end')
+
+    class dummy_acw():
+        __init__ = Func()
+        show_window = Func(result=False)
+        hide_window = Func()
 
     def test_open_completions(self):
-        # Test completions of files and attributes as well as non-completion
-        # of errors.
-        self.text.insert('1.0', 'pr')
-        self.assertTrue(self.autocomplete.open_completions(False, True, True))
+        # Test completions of files and attributes.
+        acp = self.autocomplete
+        fetch = Func(result=(['tem'],['tem', '_tem']))
+        acp.fetch_completions = fetch
+        def make_acw(): return self.dummy_acw()
+        acp._make_autocomplete_window = make_acw
+
+        self.text.insert('1.0', 'int.')
+        acp.open_completions(ac.TAB)
+        self.assertIsInstance(acp.autocompletewindow, self.dummy_acw)
         self.text.delete('1.0', 'end')
 
         # Test files.
         self.text.insert('1.0', '"t')
-        #self.assertTrue(self.autocomplete.open_completions(False, True, True))
-        self.text.delete('1.0', 'end')
-
-        # Test with blank will fail.
-        self.assertFalse(self.autocomplete.open_completions(False, True, True))
-
-        # Test with only string quote will fail.
-        self.text.insert('1.0', '"')
-        self.assertFalse(self.autocomplete.open_completions(False, True, True))
+        self.assertTrue(acp.open_completions(ac.TAB))
         self.text.delete('1.0', 'end')
 
     def test_fetch_completions(self):
@@ -174,21 +224,21 @@ def test_fetch_completions(self):
         # a small list containing non-private variables.
         # For file completion, a large list containing all files in the path,
         # and a small list containing files that do not start with '.'.
-        autocomplete = self.autocomplete
-        small, large = self.autocomplete.fetch_completions(
-                '', ac.COMPLETE_ATTRIBUTES)
+        acp = self.autocomplete
+        small, large = acp.fetch_completions(
+                '', ac.ATTRS)
         if __main__.__file__ != ac.__file__:
             self.assertNotIn('AutoComplete', small)  # See issue 36405.
 
         # Test attributes
-        s, b = autocomplete.fetch_completions('', ac.COMPLETE_ATTRIBUTES)
+        s, b = acp.fetch_completions('', ac.ATTRS)
         self.assertLess(len(small), len(large))
         self.assertTrue(all(filter(lambda x: x.startswith('_'), s)))
         self.assertTrue(any(filter(lambda x: x.startswith('_'), b)))
 
         # Test smalll should respect to __all__.
         with patch.dict('__main__.__dict__', {'__all__': ['a', 'b']}):
-            s, b = autocomplete.fetch_completions('', ac.COMPLETE_ATTRIBUTES)
+            s, b = acp.fetch_completions('', ac.ATTRS)
             self.assertEqual(s, ['a', 'b'])
             self.assertIn('__name__', b)    # From __main__.__dict__
             self.assertIn('sum', b)         # From __main__.__builtins__.__dict__
@@ -197,7 +247,7 @@ def test_fetch_completions(self):
         mock = Mock()
         mock._private = Mock()
         with patch.dict('__main__.__dict__', {'foo': mock}):
-            s, b = autocomplete.fetch_completions('foo', ac.COMPLETE_ATTRIBUTES)
+            s, b = acp.fetch_completions('foo', ac.ATTRS)
             self.assertNotIn('_private', s)
             self.assertIn('_private', b)
             self.assertEqual(s, [i for i in sorted(dir(mock)) if i[:1] != '_'])
@@ -211,36 +261,36 @@ def _listdir(path):
             return ['monty', 'python', '.hidden']
 
         with patch.object(os, 'listdir', _listdir):
-            s, b = autocomplete.fetch_completions('', ac.COMPLETE_FILES)
+            s, b = acp.fetch_completions('', ac.FILES)
             self.assertEqual(s, ['bar', 'foo'])
             self.assertEqual(b, ['.hidden', 'bar', 'foo'])
 
-            s, b = autocomplete.fetch_completions('~', ac.COMPLETE_FILES)
+            s, b = acp.fetch_completions('~', ac.FILES)
             self.assertEqual(s, ['monty', 'python'])
             self.assertEqual(b, ['.hidden', 'monty', 'python'])
 
     def test_get_entity(self):
         # Test that a name is in the namespace of sys.modules and
         # __main__.__dict__.
-        autocomplete = self.autocomplete
+        acp = self.autocomplete
         Equal = self.assertEqual
 
-        Equal(self.autocomplete.get_entity('int'), int)
+        Equal(acp.get_entity('int'), int)
 
         # Test name from sys.modules.
         mock = Mock()
         with patch.dict('sys.modules', {'tempfile': mock}):
-            Equal(autocomplete.get_entity('tempfile'), mock)
+            Equal(acp.get_entity('tempfile'), mock)
 
         # Test name from __main__.__dict__.
         di = {'foo': 10, 'bar': 20}
         with patch.dict('__main__.__dict__', {'d': di}):
-            Equal(autocomplete.get_entity('d'), di)
+            Equal(acp.get_entity('d'), di)
 
         # Test name not in namespace.
         with patch.dict('__main__.__dict__', {}):
             with self.assertRaises(NameError):
-                autocomplete.get_entity('not_exist')
+                acp.get_entity('not_exist')
 
 
 if __name__ == '__main__':
diff --git a/Misc/NEWS.d/next/IDLE/2019-08-04-17-10-01.bpo-36419.TJZqOc.rst b/Misc/NEWS.d/next/IDLE/2019-08-04-17-10-01.bpo-36419.TJZqOc.rst
new file mode 100644
index 000000000000..a44fd3b59b4f
--- /dev/null
+++ b/Misc/NEWS.d/next/IDLE/2019-08-04-17-10-01.bpo-36419.TJZqOc.rst
@@ -0,0 +1 @@
+IDLE - Refactor autocompete and improve testing.



More information about the Python-checkins mailing list
diff --git a/Lib/idlelib/autocomplete_w.py b/Lib/idlelib/autocomplete_w.py
index c24962527736..c69ab4a36836 100644
--- a/Lib/idlelib/autocomplete_w.py
+++ b/Lib/idlelib/autocomplete_w.py
@@ -6,7 +6,7 @@
 from tkinter import *
 from tkinter.ttk import Frame, Scrollbar
 
-from idlelib.autocomplete import COMPLETE_FILES, COMPLETE_ATTRIBUTES
+from idlelib.autocomplete import FILES, ATTRS
 from idlelib.multicall import MC_SHIFT
 
 HIDE_VIRTUAL_EVENT_NAME = "<<autocompletewindow-hide>>"
@@ -39,8 +39,7 @@ def __init__(self, widget):
         self.completions = None
         # A list with more completions, or None
         self.morecompletions = None
         # The completion mode. Either autocomplete.COMPLETE_ATTRIBUTES or
         # autocomplete.COMPLETE_FILES
         self.mode = None
         # The current completion start, on the text box (a string)
         self.start = None
@@ -73,8 +72,8 @@ def _change_start(self, newstart):
 
     def _binary_search(self, s):
         """Find the first index in self.completions where completions[i] is
-        greater or equal to s, or the last index if there is no such
-        one."""
+        greater or equal to s, or the last index if there is no such.
+        """
         i = 0; j = len(self.completions)
         while j > i:
             m = (i + j) // 2
@@ -87,7 +86,8 @@ def _binary_search(self, s):
     def _complete_string(self, s):
         """Assuming that s is the prefix of a string in self.completions,
         return the longest string which is a prefix of all the strings which
-        s is a prefix of them. If s is not a prefix of a string, return s."""
+        s is a prefix of them. If s is not a prefix of a string, return s.
+        """
         first = self._binary_search(s)
         if self.completions[first][:len(s)] != s:
             # There is not even one completion which s is a prefix of.
@@ -116,8 +116,10 @@ def _complete_string(self, s):
         return first_comp[:i]
 
     def _selection_changed(self):
-        """Should be called when the selection of the Listbox has changed.
-        Updates the Listbox display and calls _change_start."""
+        """Call when the selection of the Listbox has changed.
+
+        Updates the Listbox display and calls _change_start.
+        """
         cursel = int(self.listbox.curselection()[0])
 
         self.listbox.see(cursel)
@@ -153,8 +155,10 @@ def _selection_changed(self):
 
     def show_window(self, comp_lists, index, complete, mode, userWantsWin):
         """Show the autocomplete list, bind events.
-        If complete is True, complete the text, and if there is exactly one
-        matching completion, don't open a list."""
+
+        If complete is True, complete the text, and if there is exactly
+        one matching completion, don't open a list.
+        """
         # Handle the start we already have
         self.completions, self.morecompletions = comp_lists
         self.mode = mode
@@ -300,7 +304,7 @@ def keypress_event(self, event):
         if keysym != "Tab":
             self.lastkey_was_tab = False
         if (len(keysym) == 1 or keysym in ("underscore", "BackSpace")
-            or (self.mode == COMPLETE_FILES and keysym in
+            or (self.mode == FILES and keysym in
                 ("period", "minus"))) \
            and not (state & ~MC_SHIFT):
             # Normal editing of text
@@ -329,10 +333,10 @@ def keypress_event(self, event):
             self.hide_window()
             return 'break'
 
-        elif (self.mode == COMPLETE_ATTRIBUTES and keysym in
+        elif (self.mode == ATTRS and keysym in
               ("period", "space", "parenleft", "parenright", "bracketleft",
                "bracketright")) or \
-             (self.mode == COMPLETE_FILES and keysym in
+             (self.mode == FILES and keysym in
               ("slash", "backslash", "quotedbl", "apostrophe")) \
              and not (state & ~MC_SHIFT):
             # If start is a prefix of the selection, but is not '' when
@@ -340,7 +344,7 @@ def keypress_event(self, event):
             # selected completion. Anyway, close the list.
             cursel = int(self.listbox.curselection()[0])
             if self.completions[cursel][:len(self.start)] == self.start \
-               and (self.mode == COMPLETE_ATTRIBUTES or self.start):
+               and (self.mode == ATTRS or self.start):
                 self._change_start(self.completions[cursel])
             self.hide_window()
             return None
@@ -39,8 +39,7 @@ def __init__(self, widget):
         # A list with more completions, or None
         self.morecompletions = None
-        # The completion mode. Either autocomplete.COMPLETE_ATTRIBUTES or
-        # autocomplete.COMPLETE_FILES
+        # The completion mode, either autocomplete.ATTRS or .FILES.
         self.mode = None
         # The current completion start, on the text box (a string)